@@ -12,17 +12,20 @@ import logging
 import warnings
 import pandas as pd
 import geopandas as gpd
+from pathlib import Path
 
 from rex.utilities.loggers import init_mult, create_dirs, init_logger
 from rex.utilities.cli_dtypes import STR, INTLIST, INT, FLOAT
 from rex.utilities.hpc import SLURM
 from rex.utilities.utilities import get_class_properties
 
+from reV.supply_curve.extent import SupplyCurveExtent
 from reVX import __version__
 from reVX.config.least_cost_xmission import LeastCostXmissionConfig
-from reVX.least_cost_xmission.least_cost_xmission import LeastCostXmission
-from reVX.least_cost_xmission.config import TRANS_LINE_CAT, LOAD_CENTER_CAT, \
-    SINK_CAT, SUBSTATION_CAT
+from reVX.least_cost_xmission.least_cost_xmission import (LeastCostXmission,
+                                                          ReinforcedXmission)
+from reVX.least_cost_xmission.config import (TRANS_LINE_CAT, LOAD_CENTER_CAT,
+                                             SINK_CAT, SUBSTATION_CAT)
 
 TRANS_CAT_TYPES = [TRANS_LINE_CAT, LOAD_CENTER_CAT, SINK_CAT, SUBSTATION_CAT]
 
@@ -88,14 +91,12 @@ def from_config(ctx, config, verbose):
         return
 
     # Split gids over mulitple SLURM jobs
-    gids = config.sc_point_gids
     name = config.name
-    logger.info('Splitting {} SC points over {} SLURM jobs'
-                .format(len(gids), config.execution_control.nodes))
+    logger.info('Splitting SC points over {} SLURM jobs'
+                .format(config.execution_control.nodes))
     for i in range(config.execution_control.nodes):
         config.name = '{}_{}'.format(name, i)
-        config.sc_point_gids = gids[i::config.execution_control.nodes]
-        eagle(config)
+        eagle(config, start_index=i)
 
 
 @main.command()
@@ -106,6 +107,14 @@ def from_config(ctx, config, verbose):
 @click.option('--features_fpath', '-feats', required=True,
               type=click.Path(exists=True),
               help="Path to GeoPackage with transmission features")
+@click.option('--balancing_areas_fpath', '-ba', type=STR, show_default=True,
+              default=None,
+              help=("Path to Balancing areas GeoPackage. If no `None`, "
+                    "Least Cost Xmission is run with reinforcement path "
+                    "costs. Features must be substations only, and the "
+                    "substation file must contain a 'ba_str' column that "
+                    "matches the BA ID in this file for the balancing area "
+                    "containing that substation. "))
 @click.option('--capacity_class', '-cap', type=str, required=True,
               help=("Capacity class of transmission features to connect "
                     "supply curve points to"))
@@ -115,9 +124,15 @@ def from_config(ctx, config, verbose):
 @click.option('--xmission_config', '-xcfg', type=STR, show_default=True,
               default=None,
               help=("Path to Xmission config .json"))
-@click.option('--sc_point_gids', '-gids', type=INTLIST, show_default=True,
-              default=None,
-              help=("List of sc_point_gids to connect to"))
+@click.option('--min_line_length', '-mll', type=int,
+              show_default=True, default=0,
+              help=("Minimum Tie-line length."))
+@click.option('--sc_point_start_index', '-start', type=int,
+              show_default=True, default=0,
+              help=("Start index of supply curve points to run."))
+@click.option('--sc_point_step_index', '-step', type=int,
+              show_default=True, default=1,
+              help=("Step index of supply curve points to run."))
 @click.option('--nn_sinks', '-nn', type=int,
               show_default=True, default=2,
               help=("Number of nearest neighbor sinks to use for clipping "
@@ -152,10 +167,11 @@ def from_config(ctx, config, verbose):
               help=("Simplify path geometries by a value before writing to "
                     "GeoPackage."))
 @click.pass_context
-def local(ctx, cost_fpath, features_fpath, capacity_class, resolution,
-          xmission_config, sc_point_gids, nn_sinks, clipping_buffer,
-          barrier_mult, max_workers, out_dir, log_dir, verbose, save_paths,
-          radius, simplify_geo):
+def local(ctx, cost_fpath, features_fpath, balancing_areas_fpath,
+          capacity_class, resolution, xmission_config, min_line_length,
+          sc_point_start_index, sc_point_step_index, nn_sinks,
+          clipping_buffer, barrier_mult, max_workers, out_dir, log_dir,
+          verbose, save_paths, radius, simplify_geo):
     """
     Run Least Cost Xmission on local hardware
     """
@@ -169,18 +185,36 @@ def local(ctx, cost_fpath, features_fpath, capacity_class, resolution,
     create_dirs(out_dir)
     logger.info('Computing Least Cost Xmission connections and writing them {}'
                 .format(out_dir))
-    least_costs = LeastCostXmission.run(cost_fpath, features_fpath,
-                                        capacity_class,
-                                        resolution=resolution,
-                                        xmission_config=xmission_config,
-                                        sc_point_gids=sc_point_gids,
-                                        nn_sinks=nn_sinks,
-                                        clipping_buffer=clipping_buffer,
-                                        barrier_mult=barrier_mult,
-                                        max_workers=max_workers,
-                                        save_paths=save_paths,
-                                        radius=radius,
-                                        simplify_geo=simplify_geo)
+    sce = SupplyCurveExtent(cost_fpath, resolution=resolution)
+    sc_point_gids = list(sce.points.index.values)
+    sc_point_gids = sc_point_gids[sc_point_start_index::sc_point_step_index]
+    if balancing_areas_fpath is not None:
+        least_costs = ReinforcedXmission.run(cost_fpath, features_fpath,
+                                             balancing_areas_fpath,
+                                             capacity_class,
+                                             resolution=resolution,
+                                             xmission_config=xmission_config,
+                                             min_line_length=min_line_length,
+                                             sc_point_gids=sc_point_gids,
+                                             clipping_buffer=clipping_buffer,
+                                             barrier_mult=barrier_mult,
+                                             max_workers=max_workers,
+                                             save_paths=save_paths,
+                                             simplify_geo=simplify_geo)
+    else:
+        least_costs = LeastCostXmission.run(cost_fpath, features_fpath,
+                                            capacity_class,
+                                            resolution=resolution,
+                                            xmission_config=xmission_config,
+                                            min_line_length=min_line_length,
+                                            sc_point_gids=sc_point_gids,
+                                            nn_sinks=nn_sinks,
+                                            clipping_buffer=clipping_buffer,
+                                            barrier_mult=barrier_mult,
+                                            max_workers=max_workers,
+                                            save_paths=save_paths,
+                                            radius=radius,
+                                            simplify_geo=simplify_geo)
     if len(least_costs) == 0:
         logger.error('No paths found.')
         return
@@ -206,71 +240,127 @@ def local(ctx, cost_fpath, features_fpath, capacity_class, resolution,
 @click.option('--drop', '-d', default=None, type=STR, multiple=True,
               help=('Transmission feature category types to drop from '
                     'results. Options: {}'.format(", ".join(TRANS_CAT_TYPES))))
-@click.option('--out-path', '-op', type=click.Path(exists=True),
+@click.option('--out-dir', '-od', type=click.Path(exists=True),
               default='.', show_default=True,
-              help='Output path for output files. Path must exist.')
+              help='Output directory for output files. Path must exist.')
 @click.option('--simplify-geo', type=FLOAT,
               show_default=True, default=None,
               help='Simplify path geometries by a value before exporting.')
-@click.argument('gpkg_files', type=click.Path(exists=True), nargs=-1)
+@click.argument('files', type=STR, nargs=-1)
 @click.pass_context
-def merge_output(ctx, split_to_geojson, out_file, out_path, drop, simplify_geo,
-                 gpkg_files):
+def merge_output(ctx, split_to_geojson, out_file, out_dir, drop,  # noqa
+                 simplify_geo, files):
     """
-    Merge output GeoPackage files and optionally convert to GeoJSON
+    Merge output GeoPackage/CSV files and optionally convert to GeoJSON
     """
-    if len(gpkg_files) == 0:
-        click.echo('No files passed to be merged')
+    log_level = "DEBUG" if ctx.obj.get('VERBOSE') else "INFO"
+    init_logger('reVX', log_level=log_level)
+
+    if len(files) == 0:
+        logger.info('No files passed to be merged')
         return
 
+    if len(files) == 1:
+        files = sorted(Path(out_dir).glob(files[0]))
+
+    logger.debug('Merging {}'.format(files))
+
     if drop:
         for cat in drop:
             if cat not in TRANS_CAT_TYPES:
-                click.echo('--drop options must on or more of {}, received {}'
-                           .format(TRANS_CAT_TYPES, drop))
+                logger.info('--drop options must on or more of {}, received {}'
+                            .format(TRANS_CAT_TYPES, drop))
                 return
 
-    click.echo('Loading: {}'.format(", ".join(gpkg_files)))
+    logger.info('Loading: {}'.format(", ".join(files)))
     warnings.filterwarnings('ignore', category=RuntimeWarning)
-    gdf = pd.concat([gpd.read_file(f) for f in gpkg_files])
+    df = pd.concat([gpd.read_file(f) if "gpkg" in f else pd.read_csv(f)
+                    for f in files])
     warnings.filterwarnings('default', category=RuntimeWarning)
 
     if drop:
-        mask = gdf['category'].isin(drop)
-        click.echo('Dropping {} of {} total features with category(ies): {}'
-                   .format(mask.sum(), len(gdf), ", ".join(drop)))
-        gdf = gdf[~mask]
+        mask = df['category'].isin(drop)
+        logger.info('Dropping {} of {} total features with category(ies): {}'
+                    .format(mask.sum(), len(df), ", ".join(drop)))
+        df = df[~mask]
 
-    gdf = gdf.reset_index()
-    gdf = gdf[['POI Name', 'State', 'dist_km', 'sc_point_gid', 'geometry']]
+    df = df.reset_index()
 
-    if len(gdf) == 0:
-        click.echo('No transmission features to save.')
+    if len(df) == 0:
+        logger.info('No transmission features to save.')
         return
 
     if simplify_geo:
-        click.echo('Simplifying geometries by {}'.format(simplify_geo))
-        gdf.geometry = gdf.geometry.simplify(simplify_geo)
+        logger.info('Simplifying geometries by {}'.format(simplify_geo))
+        df.geometry = df.geometry.simplify(simplify_geo)
 
     if not split_to_geojson:
-        out_file = ('combo_{}'.format(gpkg_files[0])
+        out_file = ('combo_{}'.format(files[0])
                     if out_file is None else out_file)
-        out_file = os.path.join(out_path, out_file)
-        click.echo('Saving to {}'.format(out_file))
-        gdf.to_file(out_file, driver="GPKG")
+        out_file = os.path.join(out_dir, out_file)
+        logger.info('Saving to {}'.format(out_file))
+        if "gpkg" in out_file:
+            df.to_file(out_file, driver="GPKG")
+        else:
+            df.to_csv(out_file, index=False)
         return
 
     # Split out put in to GeoJSON by POI name
-    for poi in set(gdf['POI Name']):
-        outf = os.path.join(out_path,
-                            "{}_paths.geojson".format(poi.replace(' ', '_')))
-        paths = gdf[gdf['POI Name'] == poi].to_crs(epsg=4326)
-        click.echo('Writing {} paths for {} to {}'
-                   .format(len(paths), poi, outf))
-        paths.to_file(outf, driver="GeoJSON")
+    for poi in set(df['POI Name']):
+        out_file = os.path.join(out_dir,
+                                "{}_paths.geojson"
+                                .format(poi.replace(' ', '_')))
+        paths = df[df['POI Name'] == poi].to_crs(epsg=4326)
+        logger.info('Writing {} paths for {} to {}'
+                    .format(len(paths), poi, out_file))
+        paths.to_file(out_file, driver="GeoJSON")
 
 
-def get_node_cmd(config):
+@main.command()
+@click.option('--cost_fpath', '-f', required=True,
+              type=click.Path(exists=True),
+              help=("Path to GeoPackage/CSV file with calculated transmission "
+                    "costs. This file must have a 'trans_gid' column that "
+                    "will be used to merge in the reinforcement costs."))
+@click.option('--reinforcement_cost_fpath', '-r', required=True,
+              type=click.Path(exists=True),
+              help=("Path to GeoPackage/CSV file with calculated "
+                    "reinforcement costs. This file must have a 'gid' column "
+                    "that will be used to merge in the reinforcement costs."))
+@click.option('--out_file', '-of', default=None, type=STR,
+              help='Name for output GeoPackage/CSV file.')
+@click.pass_context
+def merge_reinforcement_costs(ctx, cost_fpath, reinforcement_cost_fpath,
+                              out_file):
+    """
+    Merge reinforcement costs into transmission costs.
+    """
+    log_level = "DEBUG" if ctx.obj.get('VERBOSE') else "INFO"
+    init_logger('reVX', log_level=log_level)
+
+    costs = (gpd.read_file(cost_fpath)
+             if "gpkg" in cost_fpath
+             else pd.read_csv(cost_fpath))
+    r_costs = (gpd.read_file(reinforcement_cost_fpath)
+               if "gpkg" in reinforcement_cost_fpath
+               else pd.read_csv(reinforcement_cost_fpath))
+
+    r_costs.index = r_costs.gid
+
+    logger.info("Merging reinforcement costs into transmission costs...")
+
+    r_cols = ["reinforcement_dist_km", "reinforcement_cost_per_mw"]
+    costs[r_cols] = r_costs.loc[costs["trans_gid"], r_cols].values
+
+    logger.info("Writing output to {!r}".format(out_file))
+
+    if "gpkg" in out_file:
+        costs.to_file(out_file, driver="GPKG", index=False)
+    else:
+        costs.to_csv(out_file, index=False)
+
+
+def get_node_cmd(config, start_index=0):
     """
     Get the node CLI call for Least Cost Xmission
 
@@ -289,10 +379,13 @@ def get_node_cmd(config):
             'local',
             '-cost {}'.format(SLURM.s(config.cost_fpath)),
             '-feats {}'.format(SLURM.s(config.features_fpath)),
+            '-ba {}'.format(SLURM.s(config.balancing_areas_fpath)),
             '-cap {}'.format(SLURM.s(config.capacity_class)),
             '-res {}'.format(SLURM.s(config.resolution)),
             '-xcfg {}'.format(SLURM.s(config.xmission_config)),
-            '-gids {}'.format(SLURM.s(config.sc_point_gids)),
+            '-mll {}'.format(SLURM.s(config.min_line_length)),
+            '-start {}'.format(SLURM.s(start_index)),
+            '-step {}'.format(SLURM.s(config.execution_control.nodes or 1)),
             '-nn {}'.format(SLURM.s(config.nn_sinks)),
             '-buffer {}'.format(SLURM.s(config.clipping_buffer)),
             '-bmult {}'.format(SLURM.s(config.barrier_mult)),
@@ -333,10 +426,13 @@ def run_local(ctx, config):
     ctx.invoke(local,
                cost_fpath=config.cost_fpath,
                features_fpath=config.features_fpath,
+               balancing_areas_fpath=config.balancing_areas_fpath,
                capacity_class=config.capacity_class,
                resolution=config.resolution,
                xmission_config=config.xmission_config,
-               sc_point_gids=config.sc_point_gids,
+               min_line_length=config.min_line_length,
+               sc_point_start_index=0,
+               sc_point_step_index=1,
                nn_sinks=config.nn_sinks,
                clipping_buffer=config.clipping_buffer,
                barrier_mult=config.barrier_mult,
@@ -350,7 +446,7 @@ def run_local(ctx, config):
                )
 
 
-def eagle(config):
+def eagle(config, start_index=0):
     """
     Run Least Cost Xmission on Eagle HPC.
 
@@ -361,7 +457,7 @@ def eagle(config):
     """
     init_logger('rex', log_level='DEBUG')
 
-    cmd = get_node_cmd(config)
+    cmd = get_node_cmd(config, start_index)
     name = config.name
     log_dir = config.log_directory
     stdout_path = os.path.join(log_dir, 'stdout/')
