@@ -41,8 +41,8 @@ class PlexosAggregation(BaseProfileAggregation):
     def __init__(self, plexos_nodes, rev_sc, reeds_build, cf_fpath,
                  forecast_fpath=None, build_year=2050, plexos_columns=None,
                  force_full_build=False, force_shape_map=False,
-                 plant_name_col=None, tech_tag=None, timezone='UTC',
-                 max_workers=None):
+                 plant_name_col=None, tech_tag=None, res_class=None,
+                 timezone='UTC', dset_tag=None, max_workers=None):
         """
         Parameters
         ----------
@@ -91,11 +91,21 @@ class PlexosAggregation(BaseProfileAggregation):
         tech_tag : str | None
             Optional technology tag to include as a suffix in the plexos output
             csv column headers.
+        res_class : int | None
+            Optional resource class to use to filter supply curve points.
+            For example, if res_class = 3 then only supply curve points with
+            class 3 will be kept in the sc_build table. This filters the on the
+            'class' column in the reeds_build table.
         timezone : str
             Timezone for output generation profiles. This is a string that will
             be passed to pytz.timezone() e.g. US/Pacific, US/Mountain,
             US/Central, US/Eastern, or UTC. For a list of all available
             timezones, see pytz.all_timezones
+        dset_tag : str
+            Dataset tag to append to dataset names in cf profile file. e.g. If
+            the cf profile file is a multi year file using dset_tag="-2008"
+            will enable us to select the corresponding datasets
+            (cf_mean-2008, cf_profile-2008, etc)
         max_workers : int | None
             Max workers for parallel profile aggregation. None uses all
             available workers. 1 will run in serial.
@@ -114,6 +124,8 @@ class PlexosAggregation(BaseProfileAggregation):
         self._plant_name_col = plant_name_col
         self._tech_tag = tech_tag
         self._timezone = timezone
+        self._res_class = res_class
+        self._dset_tag = dset_tag if dset_tag is not None else ""
 
         if plexos_columns is None:
             plexos_columns = tuple()
@@ -121,11 +133,18 @@ class PlexosAggregation(BaseProfileAggregation):
         self._plexos_columns += DataCleaner.PLEXOS_META_COLS
         self._plexos_columns = tuple(set(self._plexos_columns))
 
-        logger.info('Running PLEXOS aggregation for build year: {}'
-                    .format(build_year))
+        logger.info('Running {} for build year: {}'
+                    .format(self.__class__.__name__, build_year))
 
         self._sc_build = self._parse_rev_reeds(rev_sc, reeds_build,
                                                build_year=build_year)
+        if res_class is not None:
+            class_mask = self._sc_build['class'] == res_class
+            self._sc_build = self._sc_build[class_mask]
+        if self._sc_build.empty:
+            msg = 'res_class={} not found in reeds build out'.format(res_class)
+            logger.error(msg)
+            raise RuntimeError(msg)
         self._plexos_nodes = self._parse_plexos_nodes(plexos_nodes)
 
         missing = self._check_gids()
@@ -286,7 +305,7 @@ class PlexosAggregation(BaseProfileAggregation):
         rev_mask = rev_sc[join_on].isin(reeds_sc_gids)
         if not rev_mask.any():
             msg = ("There are no overlapping sc_gids between the provided reV "
-                   "supply curve table the ReEDS buildout!")
+                   "supply curve table and the ReEDS buildout!")
             logger.error(msg)
             raise RuntimeError(msg)
 
@@ -317,7 +336,7 @@ class PlexosAggregation(BaseProfileAggregation):
         ----------
         rev_sc : str | pd.DataFrame
             reV supply curve results table including SC gid, lat/lon,
-            res_gids, gid_counts. Or  path to reV supply curve table.
+            res_gids, gid_counts. Or path to reV supply curve table.
         reeds_build : str | pd.DataFrame
             ReEDS buildout with rows for built capacity (MW) at each reV SC
             point. This should have columns: reeds_year, built_capacity, and
@@ -485,6 +504,7 @@ class PlexosAggregation(BaseProfileAggregation):
             temp = RegionClassifier.run(self.sc_build, self._plexos_nodes,
                                         regions_label='plexos_id',
                                         force=self._force_shape_map)
+
             plx_node_index = temp['plexos_id'].values.astype(int)
             if any(plx_node_index < 0):
                 msg = ('Could not find a matching shape for {} supply curve '
@@ -551,7 +571,6 @@ class PlexosAggregation(BaseProfileAggregation):
 
         if len(plx_node_index.shape) == 1:
             plx_node_index = plx_node_index.reshape((len(plx_node_index), 1))
-
         return plx_node_index
 
     def make_profiles(self):
@@ -563,7 +582,7 @@ class PlexosAggregation(BaseProfileAggregation):
             (t, n) array of Plexos node generation profiles where t is the
             timeseries length and n is the number of plexos nodes.
         """
-
+        logger.info('Making profiles for {} nodes'.format(self.n_plexos_nodes))
         if self.max_workers != 1:
             profiles = self._make_profiles_parallel()
         else:
@@ -593,7 +612,8 @@ class PlexosAggregation(BaseProfileAggregation):
                                res_gids=self.available_res_gids,
                                forecast_fpath=self._forecast_fpath,
                                forecast_map=self._forecast_map,
-                               force_full_build=self._force_full_build)
+                               force_full_build=self._force_full_build,
+                               dset_tag=self._dset_tag)
                 futures[f] = i
 
             for n, f in enumerate(as_completed(futures)):
@@ -606,7 +626,7 @@ class PlexosAggregation(BaseProfileAggregation):
                 current_prog = (n + 1) // (len(futures) / 100)
                 if current_prog > progress:
                     progress = current_prog
-                    logger.info('{} % of plexos node profiles built.'
+                    logger.info('{} % of node profiles built.'
                                 .format(progress))
 
         return profiles
@@ -629,7 +649,8 @@ class PlexosAggregation(BaseProfileAggregation):
                 res_gids=self.available_res_gids,
                 forecast_fpath=self._forecast_fpath,
                 forecast_map=self._forecast_map,
-                force_full_build=self._force_full_build)
+                force_full_build=self._force_full_build,
+                dset_tag=self._dset_tag)
 
             profile, sc_gids, res_gids, gen_gids, res_built = p
             profiles[:, i] = profile
@@ -639,7 +660,7 @@ class PlexosAggregation(BaseProfileAggregation):
                             // (len(np.unique(self.node_map)) / 100))
             if current_prog > progress:
                 progress = current_prog
-                logger.info('{} % of plexos node profiles built.'
+                logger.info('{} % of node profiles built.'
                             .format(progress))
 
         return profiles
@@ -648,8 +669,8 @@ class PlexosAggregation(BaseProfileAggregation):
     def run(cls, plexos_nodes, rev_sc, reeds_build, cf_fpath,
             forecast_fpath=None, build_year=2050, plexos_columns=None,
             force_full_build=False, force_shape_map=False,
-            plant_name_col=None, tech_tag=None, timezone='UTC',
-            out_fpath=None, max_workers=None):
+            plant_name_col=None, tech_tag=None, res_class=None,
+            timezone='UTC', dset_tag=None, out_fpath=None, max_workers=None):
         """Run plexos aggregation.
 
         Parameters
@@ -701,11 +722,20 @@ class PlexosAggregation(BaseProfileAggregation):
         tech_tag : str | None
             Optional technology tag to include as a suffix in the plexos output
             csv column headers.
+        res_class : int | None
+            Optional resource class to use to filter supply curve points.
+            For example, if res_class = 3 then only supply curve points with
+            class 3 will be kept in the sc_build table.
         timezone : str
             Timezone for output generation profiles. This is a string that will
             be passed to pytz.timezone() e.g. US/Pacific, US/Mountain,
             US/Central, US/Eastern, or UTC. For a list of all available
             timezones, see pytz.all_timezones
+        dset_tag : str
+            Dataset tag to append to dataset names in cf profile file. e.g. If
+            the cf profile file is a multi year file using dset_tag="-2008"
+            will enable us to select the corresponding datasets
+            (cf_mean-2008, cf_profile-2008, etc)
         out_fpath : str, optional
             Path to .h5 file into which plant buildout should be saved. A
             plexos-formatted csv will also be written in the same directory.
@@ -732,7 +762,9 @@ class PlexosAggregation(BaseProfileAggregation):
                  force_shape_map=force_shape_map,
                  plant_name_col=plant_name_col,
                  tech_tag=tech_tag,
+                 res_class=res_class,
                  timezone=timezone,
+                 dset_tag=dset_tag,
                  max_workers=max_workers)
 
         profiles = pa.make_profiles()
@@ -885,16 +917,16 @@ class RevReedsPlexosManager:
         if agg_kwargs is None:
             agg_kwargs = {}
 
-        logger.info('Running PLEXOS aggregation with plexos nodes input: {}'
-                    .format(plexos_nodes))
-        logger.info('Running PLEXOS aggregation with reV SC input: {}'
-                    .format(rev_sc))
-        logger.info('Running PLEXOS aggregation with REEDS input: {}'
-                    .format(reeds_build))
-        logger.info('Running PLEXOS aggregation with reV Gen input: {}'
-                    .format(cf_fpath))
-        logger.info('Running PLEXOS aggregation with forecast filepath: {}'
-                    .format(forecast_fpath))
+        logger.info('Running {} with plant nodes input: {}'
+                    .format(cls.__name__, plexos_nodes))
+        logger.info('Running {} with reV SC input: {}'
+                    .format(cls.__name__, rev_sc))
+        logger.info('Running {} with REEDS input: {}'
+                    .format(cls.__name__, reeds_build))
+        logger.info('Running {} with reV Gen input: {}'
+                    .format(cls.__name__, cf_fpath))
+        logger.info('Running {} with forecast filepath: {}'
+                    .format(cls.__name__, forecast_fpath))
 
         pm = cls(plexos_nodes, rev_sc, reeds_build, cf_fpath,
                  forecast_fpath=forecast_fpath, wait=wait,
@@ -1085,4 +1117,4 @@ class RevReedsPlexosManager:
                                              chunks=(None, 100),
                                              data=profiles)
 
-        logger.info('Plexos aggregation complete!')
+        logger.info('{} complete!'.format(cls.__name__))
