@@ -12,6 +12,7 @@ from warnings import warn
 
 import numpy as np
 import geopandas as gpd
+import pandas as pd
 from pyproj.crs import CRS
 import rasterio
 from rasterio import features as rio_features
@@ -36,6 +37,12 @@ class AbstractExclusionCalculatorInterface(ABC):
         """np.array: Array representing no exclusions. """
         raise NotImplementedError
 
+    @property
+    @abstractmethod
+    def exclusion_merge_func(self):
+        """callable: Function to merge overlapping exclusion layers. """
+        raise NotImplementedError
+
     @abstractmethod
     def pre_process_regulations(self):
         """Reduce regulations to correct state and features.
@@ -231,14 +238,20 @@ class AbstractBaseExclusionsMerger(AbstractExclusionCalculatorInterface):
         regulations_df = regulations_df.set_index('FIPS')
 
         logger.info('Merging county geometries w/ local regulations')
-        s = rio_features.shapes(
+        shapes_from_raster = rio_features.shapes(
             self._fips.astype(np.int32),
             transform=cnty_fips_profile['transform']
         )
-        for p, v in s:
-            v = int(v)
-            if v in regulations_df.index:
-                regulations_df.at[v, 'geometry'] = shape(p)
+        county_regs = []
+        for polygon, fips_code in shapes_from_raster:
+            fips_code = int(fips_code)
+            if fips_code in regulations_df.index:
+                local_regs = regulations_df.loc[[fips_code]].copy()
+                local_regs['geometry'] = shape(polygon)
+                county_regs.append(local_regs)
+
+        if county_regs:
+            regulations_df = pd.concat(county_regs)
 
         regulations_df = gpd.GeoDataFrame(
             regulations_df,
@@ -449,8 +462,8 @@ class AbstractBaseExclusionsMerger(AbstractExclusionCalculatorInterface):
 
         if local_exclusions_exist and not generic_exclusions_exist:
             local_excl = self.compute_all_local_exclusions(max_workers=mw)
-            nea = self.no_exclusions_array.astype(local_excl.dtype)
-            return self._merge_exclusions(nea, local_excl)
+            # merge ensures local exclusions are clipped county boundaries
+            return self._merge_exclusions(None, local_excl)
 
         generic_exclusions = self.compute_generic_exclusions(max_workers=mw)
         local_exclusions = self.compute_all_local_exclusions(max_workers=mw)
@@ -463,15 +476,23 @@ class AbstractBaseExclusionsMerger(AbstractExclusionCalculatorInterface):
         self.pre_process_regulations()
         local_fips = self.regulations_table["FIPS"].unique()
         return self._combine_exclusions(generic_exclusions, local_exclusions,
-                                        local_fips)
+                                        local_fips, replace_existing=True)
 
-    def _combine_exclusions(self, existing, additional, cnty_fips):
+    def _combine_exclusions(self, existing, additional, cnty_fips,
+                            replace_existing=False):
         """Combine local exclusions using FIPS code"""
         if existing is None:
-            return additional
+            existing = self.no_exclusions_array.astype(additional.dtype)
 
-        local_exclusions_mask = np.isin(self._fips, cnty_fips)
-        existing[local_exclusions_mask] = additional[local_exclusions_mask]
+        local_exclusions = np.isin(self._fips, cnty_fips)
+        if replace_existing:
+            new_local_exclusions = additional[local_exclusions]
+        else:
+            new_local_exclusions = self.exclusion_merge_func(
+                existing[local_exclusions],
+                additional[local_exclusions])
+
+        existing[local_exclusions] = new_local_exclusions
         return existing
 
     @classmethod
@@ -1186,8 +1207,7 @@ def _error_or_warn(name, replace):
                .format(name))
         logger.error(msg)
         raise IOError(msg)
-    else:
-        msg = ('{} already exists and will be replaced!'
-               .format(name))
-        logger.warning(msg)
-        warn(msg)
+
+    msg = ('{} already exists and will be replaced!'.format(name))
+    logger.warning(msg)
+    warn(msg)
